﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ParallelTasks
{
    // Deklarácia podpornej triedy.
    internal class X
    {
        // Definícia automaticky implementovanej vlastnosti. 
        public int AIP { get; set; }
        // Definícia cieľovej paralelnej metódy.
        public void TargetMethod2()
        {
            Console.WriteLine("Day and time of method execution: {0}.",
                DateTime.Now);
        }
    }

    // Deklarácia podpornej triedy.
    internal class Generator
    {
        public int Next()
        {
            return new Random().Next(1, 11);
        }
    }
    
    class Program
    {
        // Definícia statickej cieľovej paralelnej metódy.
        static void TargetMethod1()
        {
            Console.WriteLine("42");
        }

        static void Main(string[] args)
        {
            #region Model1
            // Explicitná inštanciácia triedy Task. 
            Task task1 = new Task(new Action(TargetMethod1));
            // Naštartovanie spracovania paralelnej úlohy. 
            task1.Start();
            // Čakanie na dokončenie spracovania paralelnej úlohy. 
            task1.Wait();
            #endregion 

            #region Model2
            // Explicitná inštanciácia triedy Task.
            Task task2 = new Task(TargetMethod1);
            // Začatie spracovania paralelnej úlohy.
            task2.Start();
            // Čakanie na dokončenie spracovania paralelnej úlohy.
            task2.Wait(); 
            #endregion

            #region Model3
            // Explicitná inštanciácia triedy Task a jej inicializácia odkazom 
            // na inštančnú metódu pomocou lambda-výrazu.
            X e = new X();
            Task task3 = new Task(() => e.TargetMethod2());
            task3.Start();
            task3.Wait();
            #endregion

            #region Model4
            // Generovanie novej paralelnej úlohy pomocou návrhového vzoru Factory. 
            Task task4 = Task.Factory.StartNew((new X()).TargetMethod2);
            // Čakanie na dokončenie spracovania paralelnej úlohy.
            task4.Wait();
            #endregion 

            #region Model5
            // Konštrukcia paralelnej úlohy pomocou statickej metódy Run triedy Task.
            Task task5 = Task.Run(
                () =>
                {
                    Random gen = new Random();
                    int diceNumber = gen.Next(1, 7);
                    Console.WriteLine("Dice number: {0}.", diceNumber);
                }
            );
            task5.Wait();
            #endregion

            #region Model6
            // Tvorba futurity.
            Task<int> future = new Task<int>(new Func<int>(new Generator().Next));
            future.Start();
            future.Wait();
            Console.WriteLine("Number generated: {0}.", future.Result);
            #endregion
        }
    }
}
